"""fontselectiondialog.py - Miscellaneous constants.

Copyright (C) 2011-2014 Robert Pastierovic
https://launchpad.net/~just-me
"""

# -------------------------------------------------------------------------
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3 as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# -------------------------------------------------------------------------


import os

import gtk
import pygtk
import StringIO
from PIL import Image, ImageDraw, ImageFont

import constants

class FontSelectionDialog(gtk.Dialog):
    
    """Font Selection dialog."""
    
    def __init__(self, window, font_type, selected_font):
        self._window = window
        gtk.Dialog.__init__(self, 'Font Selection: ' + font_type, self, gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                                  (gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE))
        self.set_resizable(True)
        self.set_border_width(8)
        self.font_type = font_type

        hbox = gtk.HBox(False, 10)

        # list of available fonts
        sw = gtk.ScrolledWindow()
        sw.set_shadow_type(gtk.SHADOW_ETCHED_IN)
        sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        if self._window._window.preferences.get_value("hidpi") == 'True':
          self.ui_scale_factor = 2
        else:
          self.ui_scale_factor = 1
        sw.set_usize(250 * self.ui_scale_factor, 200 * self.ui_scale_factor)
        
        hbox.pack_start(sw, True, True, 0)

        store = self.create_model()

        self.treeView = gtk.TreeView(store)
        self.treeView.set_rules_hint(True)
        sw.add(self.treeView)
        
        self.treeView.set_cursor(selected_font, start_editing=True)

        self.create_columns(self.treeView)

        # font drawing
        vbox = gtk.VBox(False, 10)

        label = gtk.Label("Font Preview:")
        vbox.pack_start(label)

        self.font_image = gtk.Image()
        self.font_image.set_from_stock(gtk.STOCK_BOLD, gtk.ICON_SIZE_LARGE_TOOLBAR)
        self.get_font_preview(constants.FONTS_LIST[self.treeView.get_cursor()[0][0]][1])

        vbox.pack_start(self.font_image)
        hbox.pack_start(vbox)
        
        self.vbox.pack_start(hbox)
        self.show_all()
        self.treeView.connect("cursor-changed", self.on_cursor_changed)
        self.treeView.connect("row-activated", self.on_activated)

        # adjust scroll window
        scroll_adjustment = self.treeView.get_cursor()[0][0]/float(len(constants.FONTS_LIST))*(self.treeView.get_vadjustment().upper - self.treeView.get_vadjustment().lower)
        if scroll_adjustment > self.treeView.get_vadjustment().upper:
          scroll_adjustment = self.treeView.get_vadjustment().upper
        self.treeView.get_vadjustment().set_value(scroll_adjustment)

        self.run()
        self.destroy()


    def create_model(self):
        store = gtk.ListStore(str)
        for idx, font in enumerate(constants.FONTS_LIST, start = 0):
            store.append([font[0].replace('.ttf', '').replace('.TTF', '').replace('.otf', '').replace('.OTF', '')])
        return store

    def create_columns(self, treeView):
        rendererText = gtk.CellRendererText()
        column = gtk.TreeViewColumn("Font Name", rendererText, text=0)
        column.set_sort_column_id(0)
        treeView.append_column(column)

    def on_cursor_changed(self, widget, *args):
        self._window._window.preferences.set_value(self.font_type, constants.FONTS_LIST[widget.get_cursor()[0][0]][0])
        self.get_font_preview(constants.FONTS_LIST[widget.get_cursor()[0][0]][1])

    def on_activated(self, widget, *args):
        self._window._window.preferences.set_value(self.font_type, constants.FONTS_LIST[widget.get_cursor()[0][0]][0])
        gtk.Widget.destroy(self)

    def get_font_preview(self, font_path):
        font_image = Image.new("RGB", (200 * self.ui_scale_factor,50 * self.ui_scale_factor), "#fff")
        draw = ImageDraw.Draw(font_image)
        font = ImageFont.truetype(font_path, 20 * self.ui_scale_factor)
        draw.text((10 * self.ui_scale_factor, 10 * self.ui_scale_factor), "AaBbCc DdEeFf", font=font, fill="#000")


        pixbuf_image = pil_to_pixbuf(font_image, "#000")
        self.font_image.set_from_pixbuf(pixbuf_image)
        


def pil_to_pixbuf(PILImage, BGColor):
    """Return a pixbuf created from the PIL <image>."""

    bcolor = (int(gtk.gdk.color_parse(BGColor).red_float*255), int(gtk.gdk.color_parse(BGColor).green_float*255), int(gtk.gdk.color_parse(BGColor).blue_float*255))

    PILImage = PILImage.convert("RGBA")
    bg = Image.new("RGB", PILImage.size, bcolor)
    bg.paste(PILImage,PILImage)

    dummy_file = StringIO.StringIO()
    bg.save(dummy_file, "ppm")
    contents = dummy_file.getvalue()
    dummy_file.close()

    loader = gtk.gdk.PixbufLoader("pnm")
    loader.write(contents, len(contents))
    pixbuf = loader.get_pixbuf()
    loader.close()
    return pixbuf
